# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .deployment_status_value import DeploymentStatusValue
from .deployment_transition import DeploymentTransition


class DeploymentStatus(UniversalBaseModel):
    id: typing.Optional[str] = None
    deployment_id: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="deploymentId")] = None
    status: typing.Optional[DeploymentStatusValue] = None
    state: typing.Optional[typing.Dict[str, typing.Any]] = None
    transition: typing.Optional[DeploymentTransition] = None
    message: typing.Optional[str] = None
    retry_count: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="retryCount")] = None
    created_at: typing_extensions.Annotated[typing.Optional[dt.datetime], FieldMetadata(alias="createdAt")] = None
    updated_at: typing_extensions.Annotated[typing.Optional[dt.datetime], FieldMetadata(alias="updatedAt")] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .auto_rotate import AutoRotate
from .notification_target import NotificationTarget
from .permissions import Permissions
from .secret_store_config import SecretStoreConfig
from .virtual_account_owned_by import VirtualAccountOwnedBy


class VirtualAccountManifest(UniversalBaseModel):
    name: str = pydantic.Field()
    """
    Virtual Account Name
    """

    type: typing.Literal["virtual-account"] = pydantic.Field(default="virtual-account")
    """
    +value=virtual-account
    """

    permissions: typing.List[Permissions] = pydantic.Field()
    """
    Permissions
    """

    expiration_date: typing.Optional[str] = pydantic.Field(default=None)
    """
    Expiration Date of the Virtual Account (should be in the format yyyy-mm-dd)
    """

    auto_rotate: typing.Optional[AutoRotate] = None
    notification_target: typing.Optional[NotificationTarget] = None
    secret_store_config: typing.Optional[SecretStoreConfig] = None
    owned_by: typing_extensions.Annotated[typing.Optional[VirtualAccountOwnedBy], FieldMetadata(alias="ownedBy")] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .build_build_source import BuildBuildSource
from .build_build_spec import BuildBuildSpec


class Build(UniversalBaseModel):
    """
    Describes how we build our code into a Docker image.
    """

    type: typing.Literal["build"] = pydantic.Field(default="build")
    """
    +value=build
    """

    docker_registry: typing.Optional[str] = pydantic.Field(default=None)
    """
    FQN of the container registry. If you can't find your registry here,
    add it through the [Integrations](/integrations?tab=docker-registry) page
    """

    build_source: BuildBuildSource = pydantic.Field()
    """
    Source code location.
    """

    build_spec: BuildBuildSpec = pydantic.Field()
    """
    Instructions to build a container image out of the build source
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .virtual_mcp_server_source import VirtualMcpServerSource


class VirtualMcpServerIntegration(UniversalBaseModel):
    """
    Virtual MCP Server
    """

    type: typing.Literal["integration/mcp-server/virtual"] = pydantic.Field(default="integration/mcp-server/virtual")
    """
    +value=integration/mcp-server/virtual
    """

    name: str = pydantic.Field()
    """
    The name of the Virtual MCP Server.
    """

    description: str = pydantic.Field()
    """
    Provide a brief description of the purpose of this Virtual MCP Server.
    """

    servers: typing.List[VirtualMcpServerSource] = pydantic.Field()
    """
    List of source MCP servers to include in this virtual server.
    """

    authorized_subjects: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    List of teams/users who can use this MCP server.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from .async_service_autoscaling import AsyncServiceAutoscaling

AsyncServiceReplicas = typing.Union[float, AsyncServiceAutoscaling]



# This file was auto-generated by Fern from our API Definition.

import typing

from .git_source import GitSource
from .remote_source import RemoteSource

SparkImageBuildBuildSource = typing.Union[GitSource, RemoteSource]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .resources_devices_item import ResourcesDevicesItem
from .resources_node import ResourcesNode


class Resources(UniversalBaseModel):
    """
    Configure resource allocations, specify node constraints and capacity types to improve performance and reduce expenses. [Docs](https://docs.truefoundry.com/docs/resources)
    """

    cpu_request: float = pydantic.Field(default=0.2)
    """
    Requested CPU which determines the minimum cost incurred. The CPU usage can exceed the requested
    amount, but not the value specified in the limit. 1 CPU means 1 CPU core. Fractional CPU can be requested
    like `0.5` or `0.05`
    """

    cpu_limit: float = pydantic.Field(default=0.5)
    """
    CPU limit beyond which the usage cannot be exceeded. 1 CPU means 1 CPU core. Fractional CPU can be requested
    like `0.5`. CPU limit should be >= cpu request.
    """

    memory_request: int = pydantic.Field(default=200)
    """
    Requested memory which determines the minimum cost incurred. The unit of memory is in megabytes(MB).
    So 1 means 1 MB and 2000 means 2GB.
    """

    memory_limit: int = pydantic.Field(default=500)
    """
    Memory limit after which the application will be killed with an OOM error. The unit of memory is
    in megabytes(MB). So 1 means 1 MB and 2000 means 2GB. MemoryLimit should be greater than memory request.
    """

    ephemeral_storage_request: int = pydantic.Field(default=1000)
    """
    Requested disk storage. The unit of memory is in megabytes(MB).
    This is ephemeral storage and will be wiped out on pod restarts or eviction
    """

    ephemeral_storage_limit: int = pydantic.Field(default=2000)
    """
    Disk storage limit. The unit of memory is in megabytes(MB). Exceeding this limit will result in eviction.
    It should be greater than the request. This is ephemeral storage and will be wiped out on pod restarts or eviction
    """

    shared_memory_size: typing.Optional[int] = pydantic.Field(default=None)
    """
    Define the shared memory requirements for your workload. Machine learning libraries like Pytorch can use Shared Memory
    for inter-process communication. If you use this, we will mount a `tmpfs` backed volume at the `/dev/shm` directory.
    Any usage will also count against the workload's memory limit (`resources.memory_limit`) along with your workload's memory usage.
    If the overall usage goes above `resources.memory_limit` the user process may get killed.
    Shared Memory Size cannot be more than the defined Memory Limit for the workload.
    """

    node: typing.Optional[ResourcesNode] = pydantic.Field(default=None)
    """
    This field determines how the underlying node resource is to be utilized
    """

    devices: typing.Optional[typing.List[ResourcesDevicesItem]] = pydantic.Field(default=None)
    """
    Define custom device or accelerator requirements for your workload. We currently support NVIDIA GPUs, AWS Inferentia Accelerators, Single Host TPU Slices.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .rate_limit_unit import RateLimitUnit
from .rate_limit_when import RateLimitWhen


class RateLimitRule(UniversalBaseModel):
    """
    Rate Limit Rule
    """

    id: str = pydantic.Field()
    """
    Unique identifier for the rule
    """

    when: RateLimitWhen
    limit_to: int = pydantic.Field()
    """
    Maximum number of requests or tokens allowed
    """

    unit: RateLimitUnit

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core import enum

T_Result = typing.TypeVar("T_Result")


class McpServerOAuth2JwtSource(enum.StrEnum):
    """
    Source of the JWT token to be used for verification.
    """

    ACCESS_TOKEN = "access_token"
    ID_TOKEN = "id_token"

    def visit(self, access_token: typing.Callable[[], T_Result], id_token: typing.Callable[[], T_Result]) -> T_Result:
        if self is McpServerOAuth2JwtSource.ACCESS_TOKEN:
            return access_token()
        if self is McpServerOAuth2JwtSource.ID_TOKEN:
            return id_token()



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .param_param_type import ParamParamType


class Param(UniversalBaseModel):
    name: str = pydantic.Field()
    """
    Name of the param
    """

    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    Description of param
    """

    default: typing.Optional[str] = pydantic.Field(default=None)
    """
    Default value or placeholder
    """

    param_type: typing.Optional[ParamParamType] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class DatabricksServicePrincipalAuth(UniversalBaseModel):
    """
    You can authenticate using Databricks service principal. To generate OAuth secret and Client ID, go to Databricks workspace -> Workspace Settings -> Identity and Access -> Service Principals -> Secrets.
    """

    type: typing.Literal["service-principal"] = pydantic.Field(default="service-principal")
    """
    +value=service-principal
    """

    oauth_secret: str = pydantic.Field()
    """
    OAuth secret of the Databricks service principal.
    """

    client_id: str = pydantic.Field()
    """
    Client ID or Application ID of the Databricks service principal.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .custom_tls_settings import CustomTlsSettings
from .model_cost_metric import ModelCostMetric
from .model_type import ModelType
from .self_hosted_model_auth_data import SelfHostedModelAuthData
from .self_hosted_model_model_server import SelfHostedModelModelServer


class SelfHostedModel(UniversalBaseModel):
    """
    Self Hosted Model Integration
    """

    type: typing.Literal["integration/model/self-hosted-model"] = pydantic.Field(
        default="integration/model/self-hosted-model"
    )
    """
    +value=integration/model/self-hosted-model
    """

    name: str = pydantic.Field()
    """
    A descriptive name to identify this self-hosted model in the UI
    """

    hosted_model_name: str = pydantic.Field()
    """
    The name of the hosted model
    """

    url: str = pydantic.Field()
    """
    The URL where the model is hosted
    """

    model_server: SelfHostedModelModelServer = pydantic.Field()
    """
    The type of model server being used
    """

    tfy_application_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    +uiType=Hidden
    """

    model_types: typing.List[ModelType] = pydantic.Field()
    """
    Specify the type of the model
    """

    tls_settings: typing.Optional[CustomTlsSettings] = None
    auth_data: typing.Optional[SelfHostedModelAuthData] = pydantic.Field(default=None)
    """
    Self Hosted Model authentication data for the integration.
    """

    headers: typing.Optional[typing.Dict[str, str]] = pydantic.Field(default=None)
    """
    Self Hosted Model headers for the integration. Forwarded to the provider as is. For example: `{"Authorization": "APIKey <token>"}`
    """

    cost: typing.Optional[ModelCostMetric] = None
    authorized_subjects: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    List of subjects that are authorized to access this integration. List of user fqn in format <user_type>:<username>.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.pagination import AsyncPager, SyncPager
from ..core.request_options import RequestOptions
from ..types.empty_response import EmptyResponse
from ..types.get_ml_repo_response import GetMlRepoResponse
from ..types.list_ml_repos_response import ListMlReposResponse
from ..types.ml_repo import MlRepo
from ..types.ml_repo_manifest import MlRepoManifest
from .raw_client import AsyncRawMlReposClient, RawMlReposClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class MlReposClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawMlReposClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawMlReposClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawMlReposClient
        """
        return self._raw_client

    def create_or_update(
        self, *, manifest: MlRepoManifest, request_options: typing.Optional[RequestOptions] = None
    ) -> GetMlRepoResponse:
        """
        Creates or updates an MLRepo entity based on the provided manifest.

        Parameters
        ----------
        manifest : MlRepoManifest
            MLRepo manifest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetMlRepoResponse
            Returns the created or updated MLRepo entity based on the provided manifest.

        Examples
        --------
        from truefoundry_sdk import Collaborator, MlRepoManifest, TrueFoundry

        client = TrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        client.ml_repos.create_or_update(
            manifest=MlRepoManifest(
                name="name",
                storage_integration_fqn="storage_integration_fqn",
                collaborators=[
                    Collaborator(
                        subject="subject",
                        role_id="role_id",
                    )
                ],
            ),
        )
        """
        _response = self._raw_client.create_or_update(manifest=manifest, request_options=request_options)
        return _response.data

    def get(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> GetMlRepoResponse:
        """
        Get a ml repo by id
        Args:
            id: Unique identifier of the ml repo to get
            user_info: Authenticated user information

        Returns:
            GetMLRepoResponse: The ml repo

        Parameters
        ----------
        id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetMlRepoResponse
            Successful Response

        Examples
        --------
        from truefoundry_sdk import TrueFoundry

        client = TrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        client.ml_repos.get(
            id="id",
        )
        """
        _response = self._raw_client.get(id, request_options=request_options)
        return _response.data

    def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> EmptyResponse:
        """
        Delete a ml repo
        Args:
            id: Unique identifier of the ml repo to delete
            user_info: Authenticated user information

        Returns:
            EmptyResponse: Empty response indicating successful deletion

        Parameters
        ----------
        id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EmptyResponse
            Successful Response

        Examples
        --------
        from truefoundry_sdk import TrueFoundry

        client = TrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        client.ml_repos.delete(
            id="id",
        )
        """
        _response = self._raw_client.delete(id, request_options=request_options)
        return _response.data

    def list(
        self,
        *,
        name: typing.Optional[str] = None,
        limit: typing.Optional[int] = 100,
        offset: typing.Optional[int] = 0,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[MlRepo, ListMlReposResponse]:
        """
        List ml repos
        Args:
            filters: Filters for the ml repos
            user_info: Authenticated user information

        Returns:
            ListMLReposResponse: List of ml repos

        Parameters
        ----------
        name : typing.Optional[str]

        limit : typing.Optional[int]

        offset : typing.Optional[int]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[MlRepo, ListMlReposResponse]
            Successful Response

        Examples
        --------
        from truefoundry_sdk import TrueFoundry

        client = TrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        response = client.ml_repos.list(
            name="name",
            limit=1,
            offset=1,
        )
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        return self._raw_client.list(name=name, limit=limit, offset=offset, request_options=request_options)


class AsyncMlReposClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawMlReposClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawMlReposClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawMlReposClient
        """
        return self._raw_client

    async def create_or_update(
        self, *, manifest: MlRepoManifest, request_options: typing.Optional[RequestOptions] = None
    ) -> GetMlRepoResponse:
        """
        Creates or updates an MLRepo entity based on the provided manifest.

        Parameters
        ----------
        manifest : MlRepoManifest
            MLRepo manifest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetMlRepoResponse
            Returns the created or updated MLRepo entity based on the provided manifest.

        Examples
        --------
        import asyncio

        from truefoundry_sdk import AsyncTrueFoundry, Collaborator, MlRepoManifest

        client = AsyncTrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.ml_repos.create_or_update(
                manifest=MlRepoManifest(
                    name="name",
                    storage_integration_fqn="storage_integration_fqn",
                    collaborators=[
                        Collaborator(
                            subject="subject",
                            role_id="role_id",
                        )
                    ],
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create_or_update(manifest=manifest, request_options=request_options)
        return _response.data

    async def get(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> GetMlRepoResponse:
        """
        Get a ml repo by id
        Args:
            id: Unique identifier of the ml repo to get
            user_info: Authenticated user information

        Returns:
            GetMLRepoResponse: The ml repo

        Parameters
        ----------
        id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetMlRepoResponse
            Successful Response

        Examples
        --------
        import asyncio

        from truefoundry_sdk import AsyncTrueFoundry

        client = AsyncTrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.ml_repos.get(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(id, request_options=request_options)
        return _response.data

    async def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> EmptyResponse:
        """
        Delete a ml repo
        Args:
            id: Unique identifier of the ml repo to delete
            user_info: Authenticated user information

        Returns:
            EmptyResponse: Empty response indicating successful deletion

        Parameters
        ----------
        id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EmptyResponse
            Successful Response

        Examples
        --------
        import asyncio

        from truefoundry_sdk import AsyncTrueFoundry

        client = AsyncTrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.ml_repos.delete(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(id, request_options=request_options)
        return _response.data

    async def list(
        self,
        *,
        name: typing.Optional[str] = None,
        limit: typing.Optional[int] = 100,
        offset: typing.Optional[int] = 0,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[MlRepo, ListMlReposResponse]:
        """
        List ml repos
        Args:
            filters: Filters for the ml repos
            user_info: Authenticated user information

        Returns:
            ListMLReposResponse: List of ml repos

        Parameters
        ----------
        name : typing.Optional[str]

        limit : typing.Optional[int]

        offset : typing.Optional[int]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[MlRepo, ListMlReposResponse]
            Successful Response

        Examples
        --------
        import asyncio

        from truefoundry_sdk import AsyncTrueFoundry

        client = AsyncTrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            response = await client.ml_repos.list(
                name="name",
                limit=1,
                offset=1,
            )
            async for item in response:
                yield item

            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        return await self._raw_client.list(name=name, limit=limit, offset=offset, request_options=request_options)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from .async_processor_sidecar import AsyncProcessorSidecar
from .async_service_replicas import AsyncServiceReplicas
from .base_service import BaseService
from .rolling import Rolling
from .worker_config import WorkerConfig


class AsyncService(BaseService):
    """
    Describes the configuration for the async-service
    """

    type: typing.Literal["async-service"] = pydantic.Field(default="async-service")
    """
    +value=async-service
    """

    replicas: AsyncServiceReplicas = pydantic.Field()
    """
    Deploy multiple instances of your pods to distribute incoming traffic across them, ensuring effective load balancing.
    """

    rollout_strategy: typing.Optional[Rolling] = None
    worker_config: WorkerConfig
    sidecar: typing.Optional[AsyncProcessorSidecar] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class EnvironmentColor(UniversalBaseModel):
    color_hex: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="colorHex")] = None
    background_color_hex: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="backgroundColorHex")
    ] = None
    color: typing.Optional[str] = None
    background_color: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="backgroundColor")] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .python_build_command import PythonBuildCommand
from .python_build_python_dependencies import PythonBuildPythonDependencies


class PythonBuild(UniversalBaseModel):
    """
    Describes that we are using python to build a container image with a specific python version and pip packages installed.
    """

    type: typing.Literal["tfy-python-buildpack"] = pydantic.Field(default="tfy-python-buildpack")
    """
    +value=tfy-python-buildpack
    """

    python_version: typing.Optional[str] = pydantic.Field(default=None)
    """
    Python version to run your application. Should be one of the tags listed on [Official Python Docker Page](https://hub.docker.com/_/python)
    """

    build_context_path: str = pydantic.Field(default="./")
    """
    Build path relative to project root path.
    """

    requirements_path: typing.Optional[str] = pydantic.Field(default=None)
    """
    Path to `requirements.txt` relative to
    `Path to build context`
    """

    pip_packages: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    Define pip package requirements.
    In Python/YAML E.g. ["fastapi>=0.90,<1.0", "uvicorn"]
    """

    python_dependencies: typing.Optional[PythonBuildPythonDependencies] = pydantic.Field(default=None)
    """
    Python dependencies to install
    """

    apt_packages: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    Debian packages to install via `apt get`.
    In Python/YAML E.g. ["git", "ffmpeg", "htop"]
    """

    command: PythonBuildCommand = pydantic.Field()
    """
    Command to run when the container starts.
    Command will be set as the Entrypoint of the generated image.
    When deploying a Job, the command can be templatized by defining `params` and referencing them in command
    E.g. `python main.py --learning_rate {{learning_rate}}`
    """

    cuda_version: typing.Optional[str] = pydantic.Field(default=None)
    """
    Version of CUDA Toolkit and CUDNN to install in the image
    These combinations are based off of publically available docker images on docker hub
    You can also specify a valid tag of the form {cuda_version_number}-cudnn{cudnn_version_number}-{runtime|devel}-ubuntu{ubuntu_version}
    Refer https://hub.docker.com/r/nvidia/cuda/tags for valid set of values
    Note: We use deadsnakes ubuntu ppa to add Python that currently supports only Ubuntu 18.04, 20.04 and 22.04
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core import enum

T_Result = typing.TypeVar("T_Result")


class PatronusJudgeCriteria(enum.StrEnum):
    PATRONUS_ANSWER_REFUSAL = "patronus:answer-refusal"
    PATRONUS_NO_OPENAI_REFERENCE = "patronus:no-openai-reference"
    PATRONUS_IS_CONCISE = "patronus:is-concise"
    PATRONUS_NO_APOLOGIES = "patronus:no-apologies"
    PATRONUS_IS_POLITE = "patronus:is-polite"
    PATRONUS_IS_HELPFUL = "patronus:is-helpful"
    PATRONUS_CLINICALLY_INAPPROPRIATE_TONE = "patronus:clinically-inappropriate-tone"
    PATRONUS_NO_HARMFUL_THERAPEUTIC_GUIDANCE = "patronus:no-harmful-therapeutic-guidance"
    PATRONUS_IS_CODE = "patronus:is-code"
    PATRONUS_IS_CSV = "patronus:is-csv"
    PATRONUS_IS_JSON = "patronus:is-json"
    PATRONUS_NO_AGE_BIAS = "patronus:no-age-bias"
    PATRONUS_NO_GENDER_BIAS = "patronus:no-gender-bias"
    PATRONUS_NO_RACIAL_BIAS = "patronus:no-racial-bias"
    PATRONUS_PROMPT_INJECTION = "patronus:prompt-injection"

    def visit(
        self,
        patronus_answer_refusal: typing.Callable[[], T_Result],
        patronus_no_openai_reference: typing.Callable[[], T_Result],
        patronus_is_concise: typing.Callable[[], T_Result],
        patronus_no_apologies: typing.Callable[[], T_Result],
        patronus_is_polite: typing.Callable[[], T_Result],
        patronus_is_helpful: typing.Callable[[], T_Result],
        patronus_clinically_inappropriate_tone: typing.Callable[[], T_Result],
        patronus_no_harmful_therapeutic_guidance: typing.Callable[[], T_Result],
        patronus_is_code: typing.Callable[[], T_Result],
        patronus_is_csv: typing.Callable[[], T_Result],
        patronus_is_json: typing.Callable[[], T_Result],
        patronus_no_age_bias: typing.Callable[[], T_Result],
        patronus_no_gender_bias: typing.Callable[[], T_Result],
        patronus_no_racial_bias: typing.Callable[[], T_Result],
        patronus_prompt_injection: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is PatronusJudgeCriteria.PATRONUS_ANSWER_REFUSAL:
            return patronus_answer_refusal()
        if self is PatronusJudgeCriteria.PATRONUS_NO_OPENAI_REFERENCE:
            return patronus_no_openai_reference()
        if self is PatronusJudgeCriteria.PATRONUS_IS_CONCISE:
            return patronus_is_concise()
        if self is PatronusJudgeCriteria.PATRONUS_NO_APOLOGIES:
            return patronus_no_apologies()
        if self is PatronusJudgeCriteria.PATRONUS_IS_POLITE:
            return patronus_is_polite()
        if self is PatronusJudgeCriteria.PATRONUS_IS_HELPFUL:
            return patronus_is_helpful()
        if self is PatronusJudgeCriteria.PATRONUS_CLINICALLY_INAPPROPRIATE_TONE:
            return patronus_clinically_inappropriate_tone()
        if self is PatronusJudgeCriteria.PATRONUS_NO_HARMFUL_THERAPEUTIC_GUIDANCE:
            return patronus_no_harmful_therapeutic_guidance()
        if self is PatronusJudgeCriteria.PATRONUS_IS_CODE:
            return patronus_is_code()
        if self is PatronusJudgeCriteria.PATRONUS_IS_CSV:
            return patronus_is_csv()
        if self is PatronusJudgeCriteria.PATRONUS_IS_JSON:
            return patronus_is_json()
        if self is PatronusJudgeCriteria.PATRONUS_NO_AGE_BIAS:
            return patronus_no_age_bias()
        if self is PatronusJudgeCriteria.PATRONUS_NO_GENDER_BIAS:
            return patronus_no_gender_bias()
        if self is PatronusJudgeCriteria.PATRONUS_NO_RACIAL_BIAS:
            return patronus_no_racial_bias()
        if self is PatronusJudgeCriteria.PATRONUS_PROMPT_INJECTION:
            return patronus_prompt_injection()



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .infer_method_name import InferMethodName


class SklearnModelSchema(UniversalBaseModel):
    infer_method_name: InferMethodName = pydantic.Field()
    """
    Name of the method used for inference
    """

    inputs: typing.List[typing.Dict[str, typing.Any]] = pydantic.Field()
    """
    Schema of the input
    """

    outputs: typing.List[typing.Dict[str, typing.Any]] = pydantic.Field()
    """
    Schema of the output
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class AmqpInputConfig(UniversalBaseModel):
    """
    Describes the configuration for the input AMQP worker
    """

    type: typing.Literal["amqp"] = pydantic.Field(default="amqp")
    """
    +value=amqp
    """

    url: str = pydantic.Field()
    """
    AMQP Queue URL of Subscriber
    """

    queue_name: str = pydantic.Field()
    """
    AMQP Queue Name
    """

    wait_time_seconds: int = pydantic.Field(default=5)
    """
    Wait timeout for long polling.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class Autoshutdown(UniversalBaseModel):
    wait_time: int = pydantic.Field(default=900)
    """
    The period to wait after the last received request before scaling the replicas to 0. This value should be high enough to allow for the replicas of the service to come up to avoid premature scaling down.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .mcp_server_o_auth2jwt_source import McpServerOAuth2JwtSource


class McpServerOAuth2(UniversalBaseModel):
    """
    OAuth2
    """

    type: typing.Literal["oauth2"] = pydantic.Field(default="oauth2")
    """
    OAuth2 authentication
    """

    authorization_url: str = pydantic.Field()
    """
    URL for the authorization request
    """

    token_url: str = pydantic.Field()
    """
    The endpoint to exchange auth code for tokens.
    """

    client_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    client ID for OAuth2 or the TrueFoundry secret FQN containing the client ID.
    """

    client_secret: typing.Optional[str] = pydantic.Field(default=None)
    """
    Client secret or the TrueFoundry secret FQN containing the client secret for OAuth2.
    """

    registration_url: typing.Optional[str] = pydantic.Field(default=None)
    """
    URL for dynamic client registration (RFC 7591). If provided, client credentials will be obtained automatically using the Dynamic Client Registration (DCR) process.
    """

    code_challenge_methods_supported: typing.Optional[typing.List[typing.Literal["S256"]]] = pydantic.Field(
        default=None
    )
    """
    List of supported PKCE code challenge methods (S256 only)
    """

    jwt_source: McpServerOAuth2JwtSource = pydantic.Field()
    """
    Source of the JWT token to be used for verification.
    """

    scopes: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    List of scopes to request from the OAuth2 provider.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .environment_color import EnvironmentColor
from .environment_manifest import EnvironmentManifest
from .environment_optimize_for import EnvironmentOptimizeFor
from .subject import Subject


class Environment(UniversalBaseModel):
    id: typing.Optional[str] = None
    name: str
    priority: float
    color: EnvironmentColor
    tenant_name: typing_extensions.Annotated[str, FieldMetadata(alias="tenantName")]
    created_by_subject: typing_extensions.Annotated[Subject, FieldMetadata(alias="createdBySubject")]
    is_production: typing_extensions.Annotated[bool, FieldMetadata(alias="isProduction")]
    optimize_for: typing_extensions.Annotated[
        typing.Optional[EnvironmentOptimizeFor], FieldMetadata(alias="optimizeFor")
    ] = None
    manifest: EnvironmentManifest
    created_by: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="createdBy")] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class Uv(UniversalBaseModel):
    """
    Use `uv` to setup env.
    Your build context root must contain `pyproject.toml` and `uv.lock`
    """

    type: typing.Literal["uv"] = pydantic.Field(default="uv")
    """
    +value=uv
    """

    uv_version: typing.Optional[str] = pydantic.Field(default="latest")
    """
    UV version to use
    """

    sync_options: typing.Optional[str] = pydantic.Field(default=None)
    """
    Sync options to pass to uv command
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel, update_forward_refs
from ..core.serialization import FieldMetadata
from .i_change_operation import IChangeOperation


class IChange(UniversalBaseModel):
    type: IChangeOperation
    key: str
    value: typing.Optional[typing.Dict[str, typing.Any]] = None
    old_value: typing_extensions.Annotated[
        typing.Optional[typing.Dict[str, typing.Any]], FieldMetadata(alias="oldValue")
    ] = None
    changes: typing.Optional[typing.List["IChange"]] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


update_forward_refs(IChange)



# This file was auto-generated by Fern from our API Definition.

import typing

from .pip import Pip
from .poetry import Poetry
from .uv import Uv

PythonBuildPythonDependencies = typing.Union[Pip, Uv, Poetry]



# This file was auto-generated by Fern from our API Definition.

import typing

from .secret_mount import SecretMount
from .string_data_mount import StringDataMount
from .volume_mount import VolumeMount

ContainerTaskConfigMountsItem = typing.Union[SecretMount, StringDataMount, VolumeMount]



# This file was auto-generated by Fern from our API Definition.

import typing

from .container_task_config import ContainerTaskConfig
from .py_spark_task_config import PySparkTaskConfig
from .python_task_config import PythonTaskConfig

FlyteTaskCustomTruefoundry = typing.Union[PythonTaskConfig, ContainerTaskConfig, PySparkTaskConfig]



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core import enum

T_Result = typing.TypeVar("T_Result")


class DeploymentTransition(enum.StrEnum):
    BUILDING = "BUILDING"
    DEPLOYING = "DEPLOYING"
    REUSING_EXISTING_BUILD = "REUSING_EXISTING_BUILD"
    COMPONENTS_DEPLOYING = "COMPONENTS_DEPLOYING"
    WAITING = "WAITING"
    EMPTY = ""

    def visit(
        self,
        building: typing.Callable[[], T_Result],
        deploying: typing.Callable[[], T_Result],
        reusing_existing_build: typing.Callable[[], T_Result],
        components_deploying: typing.Callable[[], T_Result],
        waiting: typing.Callable[[], T_Result],
        empty: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is DeploymentTransition.BUILDING:
            return building()
        if self is DeploymentTransition.DEPLOYING:
            return deploying()
        if self is DeploymentTransition.REUSING_EXISTING_BUILD:
            return reusing_existing_build()
        if self is DeploymentTransition.COMPONENTS_DEPLOYING:
            return components_deploying()
        if self is DeploymentTransition.WAITING:
            return waiting()
        if self is DeploymentTransition.EMPTY:
            return empty()



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class TrueFoundryInteractiveLogin(UniversalBaseModel):
    """
    Login with truefoundry
    """

    type: typing.Literal["truefoundry_oauth"] = pydantic.Field(default="truefoundry_oauth")
    """
    +value=truefoundry_oauth
    """

    bypass_auth_paths: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    List of paths that will bypass auth.
    needs to start with a forward slash(/) and should not contain wildcards(*)
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .retry_config import RetryConfig


class LatencyBasedLoadBalanceTarget(UniversalBaseModel):
    """
    Latency-based Target
    """

    target: str = pydantic.Field()
    """
    Target model or provider FQN
    """

    retry_config: typing.Optional[RetryConfig] = None
    fallback_status_codes: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    Status Codes for which the request will fallback to other targets. If the status code is not present in fallback_status_codes, it fails immediately.
    """

    fallback_candidate: typing.Optional[bool] = pydantic.Field(default=True)
    """
    Whether this target is a fallback candidate.  If set to false, this model will not be considered as a fallback option for targets of this load-balance-rule
    """

    override_params: typing.Optional[typing.Dict[str, typing.Any]] = pydantic.Field(default=None)
    """
    Optional parameters to override in the request
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core import enum

T_Result = typing.TypeVar("T_Result")


class PolicyEntityTypes(enum.StrEnum):
    SERVICE = "service"
    ASYNC_SERVICE = "async-service"
    JOB = "job"
    NOTEBOOK = "notebook"
    SSH_SERVER = "ssh-server"
    WORKFLOW = "workflow"
    HELM = "helm"

    def visit(
        self,
        service: typing.Callable[[], T_Result],
        async_service: typing.Callable[[], T_Result],
        job: typing.Callable[[], T_Result],
        notebook: typing.Callable[[], T_Result],
        ssh_server: typing.Callable[[], T_Result],
        workflow: typing.Callable[[], T_Result],
        helm: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is PolicyEntityTypes.SERVICE:
            return service()
        if self is PolicyEntityTypes.ASYNC_SERVICE:
            return async_service()
        if self is PolicyEntityTypes.JOB:
            return job()
        if self is PolicyEntityTypes.NOTEBOOK:
            return notebook()
        if self is PolicyEntityTypes.SSH_SERVER:
            return ssh_server()
        if self is PolicyEntityTypes.WORKFLOW:
            return workflow()
        if self is PolicyEntityTypes.HELM:
            return helm()



# This file was auto-generated by Fern from our API Definition.

import typing

from .artifact_manifest import ArtifactManifest
from .chat_prompt_manifest import ChatPromptManifest
from .model_manifest import ModelManifest

BaseArtifactVersionManifest = typing.Union[ModelManifest, ChatPromptManifest, ArtifactManifest]



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.pagination import AsyncPager, SyncPager
from ..core.request_options import RequestOptions
from ..types.activate_user_response import ActivateUserResponse
from ..types.change_password_response import ChangePasswordResponse
from ..types.deactivate_user_response import DeactivateUserResponse
from ..types.delete_user_response import DeleteUserResponse
from ..types.get_user_resources_response import GetUserResourcesResponse
from ..types.get_user_response import GetUserResponse
from ..types.get_user_teams_response import GetUserTeamsResponse
from ..types.invite_user_response import InviteUserResponse
from ..types.list_users_response import ListUsersResponse
from ..types.register_users_response import RegisterUsersResponse
from ..types.update_user_roles_response import UpdateUserRolesResponse
from ..types.user import User
from .raw_client import AsyncRawUsersClient, RawUsersClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class UsersClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawUsersClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawUsersClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawUsersClient
        """
        return self._raw_client

    def list(
        self,
        *,
        limit: typing.Optional[int] = 100,
        offset: typing.Optional[int] = 0,
        query: typing.Optional[str] = None,
        show_invalid_users: typing.Optional[bool] = None,
        include_virtual_accounts: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[User, ListUsersResponse]:
        """
        List all users of tenant filtered by query and showInvalidUsers. Pagination is available based on query parameters.

        Parameters
        ----------
        limit : typing.Optional[int]
            Number of items per page

        offset : typing.Optional[int]
            Number of items to skip

        query : typing.Optional[str]

        show_invalid_users : typing.Optional[bool]
            Show Deactivated users

        include_virtual_accounts : typing.Optional[str]
            Include virtual accounts

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[User, ListUsersResponse]
            Returns all users of tenant and also the response includes paginated data.

        Examples
        --------
        from truefoundry_sdk import TrueFoundry

        client = TrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        response = client.users.list(
            limit=10,
            offset=0,
            query="query",
            show_invalid_users=True,
            include_virtual_accounts="includeVirtualAccounts",
        )
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        return self._raw_client.list(
            limit=limit,
            offset=offset,
            query=query,
            show_invalid_users=show_invalid_users,
            include_virtual_accounts=include_virtual_accounts,
            request_options=request_options,
        )

    def pre_register_users(
        self,
        *,
        email: str,
        send_invite_email: typing.Optional[bool] = False,
        skip_if_user_exists: typing.Optional[bool] = False,
        dry_run: typing.Optional[bool] = False,
        accept_invite_client_url: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> RegisterUsersResponse:
        """
        This endpoint allows tenant administrators to register users within their tenant.

        Parameters
        ----------
        email : str
            Email of the user

        send_invite_email : typing.Optional[bool]
            Send invite email if user does not exist

        skip_if_user_exists : typing.Optional[bool]
            Fail if user exists

        dry_run : typing.Optional[bool]
            Dry run

        accept_invite_client_url : typing.Optional[str]
            Url to redirect when invite is accepted

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RegisterUsersResponse
            The users have been successfully registered.

        Examples
        --------
        from truefoundry_sdk import TrueFoundry

        client = TrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        client.users.pre_register_users(
            email="email",
        )
        """
        _response = self._raw_client.pre_register_users(
            email=email,
            send_invite_email=send_invite_email,
            skip_if_user_exists=skip_if_user_exists,
            dry_run=dry_run,
            accept_invite_client_url=accept_invite_client_url,
            request_options=request_options,
        )
        return _response.data

    def update_roles(
        self,
        *,
        email: str,
        roles: typing.Sequence[str],
        resource_type: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateUserRolesResponse:
        """
        This endpoint allows tenant administrators to update the roles of a user within their tenant.

        Parameters
        ----------
        email : str
            Email of the user

        roles : typing.Sequence[str]
            Role names for the user

        resource_type : typing.Optional[str]
            Resource Type

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateUserRolesResponse
            The user roles have been successfully updated.

        Examples
        --------
        from truefoundry_sdk import TrueFoundry

        client = TrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        client.users.update_roles(
            email="email",
            roles=["roles"],
        )
        """
        _response = self._raw_client.update_roles(
            email=email, roles=roles, resource_type=resource_type, request_options=request_options
        )
        return _response.data

    def get(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> GetUserResponse:
        """
        Get User associated with provided User id

        Parameters
        ----------
        id : str
            User Id

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetUserResponse
            Returns the User associated with provided User id

        Examples
        --------
        from truefoundry_sdk import TrueFoundry

        client = TrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        client.users.get(
            id="id",
        )
        """
        _response = self._raw_client.get(id, request_options=request_options)
        return _response.data

    def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> DeleteUserResponse:
        """
        Delete user if they are not a collaborator in any resource and not part of any team other than everyone.

        Parameters
        ----------
        id : str
            User Id

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DeleteUserResponse
            User has been successfully deleted.

        Examples
        --------
        from truefoundry_sdk import TrueFoundry

        client = TrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        client.users.delete(
            id="id",
        )
        """
        _response = self._raw_client.delete(id, request_options=request_options)
        return _response.data

    def invite_user(
        self, *, accept_invite_client_url: str, email: str, request_options: typing.Optional[RequestOptions] = None
    ) -> InviteUserResponse:
        """
        Invite a user to the tenant

        Parameters
        ----------
        accept_invite_client_url : str
            Url to redirect when invite is accepted

        email : str
            Email of user

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        InviteUserResponse
            User has been successfully invited.

        Examples
        --------
        from truefoundry_sdk import TrueFoundry

        client = TrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        client.users.invite_user(
            accept_invite_client_url="<control plane url>/invite-accept",
            email="email",
        )
        """
        _response = self._raw_client.invite_user(
            accept_invite_client_url=accept_invite_client_url, email=email, request_options=request_options
        )
        return _response.data

    def deactivate(
        self, *, email: str, request_options: typing.Optional[RequestOptions] = None
    ) -> DeactivateUserResponse:
        """
        Deactivate user associated with the provided email within the tenant.

        Parameters
        ----------
        email : str
            Email of the user

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DeactivateUserResponse
            User has been successfully deactivated.

        Examples
        --------
        from truefoundry_sdk import TrueFoundry

        client = TrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        client.users.deactivate(
            email="email",
        )
        """
        _response = self._raw_client.deactivate(email=email, request_options=request_options)
        return _response.data

    def activate(self, *, email: str, request_options: typing.Optional[RequestOptions] = None) -> ActivateUserResponse:
        """
        Activate user associated with the provided email within the tenant.

        Parameters
        ----------
        email : str
            Email of the user

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ActivateUserResponse
            User has been successfully activated.

        Examples
        --------
        from truefoundry_sdk import TrueFoundry

        client = TrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        client.users.activate(
            email="email",
        )
        """
        _response = self._raw_client.activate(email=email, request_options=request_options)
        return _response.data

    def change_password(
        self,
        *,
        login_id: str,
        new_password: str,
        old_password: str,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ChangePasswordResponse:
        """
        Change password for the authenticated user. Requires clientId and loginId in the request body.

        Parameters
        ----------
        login_id : str
            login id of the user(email)

        new_password : str
            New password

        old_password : str
            Old password

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ChangePasswordResponse
            Password has been changed successfully.

        Examples
        --------
        from truefoundry_sdk import TrueFoundry

        client = TrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        client.users.change_password(
            login_id="loginId",
            new_password="newPassword",
            old_password="oldPassword",
        )
        """
        _response = self._raw_client.change_password(
            login_id=login_id, new_password=new_password, old_password=old_password, request_options=request_options
        )
        return _response.data

    def get_resources(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetUserResourcesResponse:
        """
        Get all resources associated with a user.

        Parameters
        ----------
        id : str
            User Id

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetUserResourcesResponse
            Returns all resources for the user.

        Examples
        --------
        from truefoundry_sdk import TrueFoundry

        client = TrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        client.users.get_resources(
            id="id",
        )
        """
        _response = self._raw_client.get_resources(id, request_options=request_options)
        return _response.data

    def get_teams(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> GetUserTeamsResponse:
        """
        Get all manual teams associated with a user.

        Parameters
        ----------
        id : str
            User Id

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetUserTeamsResponse
            Returns all manual teams for the user.

        Examples
        --------
        from truefoundry_sdk import TrueFoundry

        client = TrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        client.users.get_teams(
            id="id",
        )
        """
        _response = self._raw_client.get_teams(id, request_options=request_options)
        return _response.data


class AsyncUsersClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawUsersClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawUsersClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawUsersClient
        """
        return self._raw_client

    async def list(
        self,
        *,
        limit: typing.Optional[int] = 100,
        offset: typing.Optional[int] = 0,
        query: typing.Optional[str] = None,
        show_invalid_users: typing.Optional[bool] = None,
        include_virtual_accounts: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[User, ListUsersResponse]:
        """
        List all users of tenant filtered by query and showInvalidUsers. Pagination is available based on query parameters.

        Parameters
        ----------
        limit : typing.Optional[int]
            Number of items per page

        offset : typing.Optional[int]
            Number of items to skip

        query : typing.Optional[str]

        show_invalid_users : typing.Optional[bool]
            Show Deactivated users

        include_virtual_accounts : typing.Optional[str]
            Include virtual accounts

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[User, ListUsersResponse]
            Returns all users of tenant and also the response includes paginated data.

        Examples
        --------
        import asyncio

        from truefoundry_sdk import AsyncTrueFoundry

        client = AsyncTrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            response = await client.users.list(
                limit=10,
                offset=0,
                query="query",
                show_invalid_users=True,
                include_virtual_accounts="includeVirtualAccounts",
            )
            async for item in response:
                yield item

            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        return await self._raw_client.list(
            limit=limit,
            offset=offset,
            query=query,
            show_invalid_users=show_invalid_users,
            include_virtual_accounts=include_virtual_accounts,
            request_options=request_options,
        )

    async def pre_register_users(
        self,
        *,
        email: str,
        send_invite_email: typing.Optional[bool] = False,
        skip_if_user_exists: typing.Optional[bool] = False,
        dry_run: typing.Optional[bool] = False,
        accept_invite_client_url: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> RegisterUsersResponse:
        """
        This endpoint allows tenant administrators to register users within their tenant.

        Parameters
        ----------
        email : str
            Email of the user

        send_invite_email : typing.Optional[bool]
            Send invite email if user does not exist

        skip_if_user_exists : typing.Optional[bool]
            Fail if user exists

        dry_run : typing.Optional[bool]
            Dry run

        accept_invite_client_url : typing.Optional[str]
            Url to redirect when invite is accepted

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RegisterUsersResponse
            The users have been successfully registered.

        Examples
        --------
        import asyncio

        from truefoundry_sdk import AsyncTrueFoundry

        client = AsyncTrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.users.pre_register_users(
                email="email",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.pre_register_users(
            email=email,
            send_invite_email=send_invite_email,
            skip_if_user_exists=skip_if_user_exists,
            dry_run=dry_run,
            accept_invite_client_url=accept_invite_client_url,
            request_options=request_options,
        )
        return _response.data

    async def update_roles(
        self,
        *,
        email: str,
        roles: typing.Sequence[str],
        resource_type: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateUserRolesResponse:
        """
        This endpoint allows tenant administrators to update the roles of a user within their tenant.

        Parameters
        ----------
        email : str
            Email of the user

        roles : typing.Sequence[str]
            Role names for the user

        resource_type : typing.Optional[str]
            Resource Type

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateUserRolesResponse
            The user roles have been successfully updated.

        Examples
        --------
        import asyncio

        from truefoundry_sdk import AsyncTrueFoundry

        client = AsyncTrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.users.update_roles(
                email="email",
                roles=["roles"],
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update_roles(
            email=email, roles=roles, resource_type=resource_type, request_options=request_options
        )
        return _response.data

    async def get(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> GetUserResponse:
        """
        Get User associated with provided User id

        Parameters
        ----------
        id : str
            User Id

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetUserResponse
            Returns the User associated with provided User id

        Examples
        --------
        import asyncio

        from truefoundry_sdk import AsyncTrueFoundry

        client = AsyncTrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.users.get(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(id, request_options=request_options)
        return _response.data

    async def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> DeleteUserResponse:
        """
        Delete user if they are not a collaborator in any resource and not part of any team other than everyone.

        Parameters
        ----------
        id : str
            User Id

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DeleteUserResponse
            User has been successfully deleted.

        Examples
        --------
        import asyncio

        from truefoundry_sdk import AsyncTrueFoundry

        client = AsyncTrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.users.delete(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(id, request_options=request_options)
        return _response.data

    async def invite_user(
        self, *, accept_invite_client_url: str, email: str, request_options: typing.Optional[RequestOptions] = None
    ) -> InviteUserResponse:
        """
        Invite a user to the tenant

        Parameters
        ----------
        accept_invite_client_url : str
            Url to redirect when invite is accepted

        email : str
            Email of user

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        InviteUserResponse
            User has been successfully invited.

        Examples
        --------
        import asyncio

        from truefoundry_sdk import AsyncTrueFoundry

        client = AsyncTrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.users.invite_user(
                accept_invite_client_url="<control plane url>/invite-accept",
                email="email",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.invite_user(
            accept_invite_client_url=accept_invite_client_url, email=email, request_options=request_options
        )
        return _response.data

    async def deactivate(
        self, *, email: str, request_options: typing.Optional[RequestOptions] = None
    ) -> DeactivateUserResponse:
        """
        Deactivate user associated with the provided email within the tenant.

        Parameters
        ----------
        email : str
            Email of the user

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DeactivateUserResponse
            User has been successfully deactivated.

        Examples
        --------
        import asyncio

        from truefoundry_sdk import AsyncTrueFoundry

        client = AsyncTrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.users.deactivate(
                email="email",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.deactivate(email=email, request_options=request_options)
        return _response.data

    async def activate(
        self, *, email: str, request_options: typing.Optional[RequestOptions] = None
    ) -> ActivateUserResponse:
        """
        Activate user associated with the provided email within the tenant.

        Parameters
        ----------
        email : str
            Email of the user

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ActivateUserResponse
            User has been successfully activated.

        Examples
        --------
        import asyncio

        from truefoundry_sdk import AsyncTrueFoundry

        client = AsyncTrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.users.activate(
                email="email",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.activate(email=email, request_options=request_options)
        return _response.data

    async def change_password(
        self,
        *,
        login_id: str,
        new_password: str,
        old_password: str,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ChangePasswordResponse:
        """
        Change password for the authenticated user. Requires clientId and loginId in the request body.

        Parameters
        ----------
        login_id : str
            login id of the user(email)

        new_password : str
            New password

        old_password : str
            Old password

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ChangePasswordResponse
            Password has been changed successfully.

        Examples
        --------
        import asyncio

        from truefoundry_sdk import AsyncTrueFoundry

        client = AsyncTrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.users.change_password(
                login_id="loginId",
                new_password="newPassword",
                old_password="oldPassword",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.change_password(
            login_id=login_id, new_password=new_password, old_password=old_password, request_options=request_options
        )
        return _response.data

    async def get_resources(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetUserResourcesResponse:
        """
        Get all resources associated with a user.

        Parameters
        ----------
        id : str
            User Id

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetUserResourcesResponse
            Returns all resources for the user.

        Examples
        --------
        import asyncio

        from truefoundry_sdk import AsyncTrueFoundry

        client = AsyncTrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.users.get_resources(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_resources(id, request_options=request_options)
        return _response.data

    async def get_teams(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetUserTeamsResponse:
        """
        Get all manual teams associated with a user.

        Parameters
        ----------
        id : str
            User Id

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetUserTeamsResponse
            Returns all manual teams for the user.

        Examples
        --------
        import asyncio

        from truefoundry_sdk import AsyncTrueFoundry

        client = AsyncTrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.users.get_teams(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_teams(id, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core import enum

T_Result = typing.TypeVar("T_Result")


class RateLimitUnit(enum.StrEnum):
    """
    Unit for rate limit calculation
    """

    REQUESTS_PER_DAY = "requests_per_day"
    TOKENS_PER_MINUTE = "tokens_per_minute"
    REQUESTS_PER_MINUTE = "requests_per_minute"
    TOKENS_PER_DAY = "tokens_per_day"
    REQUESTS_PER_HOUR = "requests_per_hour"
    TOKENS_PER_HOUR = "tokens_per_hour"

    def visit(
        self,
        requests_per_day: typing.Callable[[], T_Result],
        tokens_per_minute: typing.Callable[[], T_Result],
        requests_per_minute: typing.Callable[[], T_Result],
        tokens_per_day: typing.Callable[[], T_Result],
        requests_per_hour: typing.Callable[[], T_Result],
        tokens_per_hour: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is RateLimitUnit.REQUESTS_PER_DAY:
            return requests_per_day()
        if self is RateLimitUnit.TOKENS_PER_MINUTE:
            return tokens_per_minute()
        if self is RateLimitUnit.REQUESTS_PER_MINUTE:
            return requests_per_minute()
        if self is RateLimitUnit.TOKENS_PER_DAY:
            return tokens_per_day()
        if self is RateLimitUnit.REQUESTS_PER_HOUR:
            return requests_per_hour()
        if self is RateLimitUnit.TOKENS_PER_HOUR:
            return tokens_per_hour()



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class RateLimitWhen(UniversalBaseModel):
    """
    Rule Conditions
    """

    subjects: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    List of subjects that this rule applies to
    """

    models: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    List of models that this rule applies to
    """

    metadata: typing.Optional[typing.Dict[str, str]] = pydantic.Field(default=None)
    """
    Metadata key-value pairs that this rule applies to
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .open_ai_moderations_guardrail_config_category_thresholds_value import (
    OpenAiModerationsGuardrailConfigCategoryThresholdsValue,
)
from .openai_api_key_auth import OpenaiApiKeyAuth


class OpenAiModerationsGuardrailConfig(UniversalBaseModel):
    """
    OpenAI Moderation
    """

    name: str = pydantic.Field()
    """
    The name of the Guardrail Config.
    """

    type: typing.Literal["integration/guardrail-config/openai-moderations"] = pydantic.Field(
        default="integration/guardrail-config/openai-moderations"
    )
    """
    +value=integration/guardrail-config/openai-moderations
    +sort=50
    +uiType=Hidden
    """

    base_url: typing.Optional[str] = pydantic.Field(default=None)
    """
    Optional custom base URL for OpenAI API. If not provided, the default base URL will be used.
    """

    operation: typing.Optional[typing.Literal["validate"]] = pydantic.Field(default=None)
    """
    The operation type for this guardrail. OpenAI Moderation guardrails can only be used for validation.
    """

    auth_data: OpenaiApiKeyAuth
    model: str = pydantic.Field(default="omni-moderation-latest")
    """
    The model to use for the OpenAI Moderation API.
    """

    category_thresholds: typing.Optional[typing.Dict[str, OpenAiModerationsGuardrailConfigCategoryThresholdsValue]] = (
        pydantic.Field(default=None)
    )
    """
    Confidence thresholds (0.0-1.0) for each content category. Content scoring above the threshold will be flagged. Lower values are more strict
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core import enum

T_Result = typing.TypeVar("T_Result")


class CustomGuardrailConfigOperation(enum.StrEnum):
    """
    The operation type to use for the Guardrail. Validate guardrails are used to validate requests and mutate can validate as well as mutate requests.
    Validate guardrails are run in parallel while mutate guardrails are run sequentially.
    """

    VALIDATE = "validate"
    MUTATE = "mutate"

    def visit(self, validate: typing.Callable[[], T_Result], mutate: typing.Callable[[], T_Result]) -> T_Result:
        if self is CustomGuardrailConfigOperation.VALIDATE:
            return validate()
        if self is CustomGuardrailConfigOperation.MUTATE:
            return mutate()



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class GcpTpu(UniversalBaseModel):
    type: typing.Literal["gcp_tpu"] = pydantic.Field(default="gcp_tpu")
    """
    +value=gcp_tpu
    """

    name: str = pydantic.Field()
    """
    Name of the TPU Type. One of
      - `tpu-v4-podslice` (TPU v4, ct4p)
      - `tpu-v5-lite-device` (TPU v5e, ct5l)
      - `tpu-v5-lite-podslice`  (TPU v5e, ct5lp)
      - `tpu-v5p-slice` (TPU v5p, ct5p)
    """

    topology: str = pydantic.Field()
    """
    Topology of the TPU slices. Currently only single-host topology is supported.
     Please refer to [TPUs on GKE docs](https://cloud.google.com/kubernetes-engine/docs/concepts/tpus#plan-tpu-configuration)
     Allowed Values:
       - `2x2x1` for `tpu-v4-podslice`
       - One of `1x1`, `2x2`, `2x4` for `tpu-v5-lite-device` and `tpu-v5-lite-podslice`
       - `2x2x1` for `tpu-v5p-slice`
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from .docker_file_build import DockerFileBuild
from .python_build import PythonBuild

BuildBuildSpec = typing.Union[DockerFileBuild, PythonBuild]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .python_task_config_image import PythonTaskConfigImage
from .python_task_config_mounts_item import PythonTaskConfigMountsItem
from .resources import Resources


class PythonTaskConfig(UniversalBaseModel):
    """
    Describes the configuration for the python function task
    """

    type: typing.Literal["python-task-config"] = pydantic.Field(default="python-task-config")
    """
    +value=python-task-config
    """

    image: PythonTaskConfigImage = pydantic.Field()
    """
    Specify the image spec for the task
    """

    env: typing.Optional[typing.Dict[str, typing.Optional[str]]] = pydantic.Field(default=None)
    """
    Configure environment variables to be injected in the task either as plain text or secrets. [Docs](https://docs.truefoundry.com/docs/env-variables)
    """

    resources: typing.Optional[Resources] = None
    mounts: typing.Optional[typing.List[PythonTaskConfigMountsItem]] = pydantic.Field(default=None)
    """
    Configure data to be mounted to Workflow pod(s) as a volume.
    """

    service_account: typing.Optional[str] = pydantic.Field(default=None)
    """
    Service Account
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.request_options import RequestOptions
from .raw_client import AsyncRawWorkflowsClient, RawWorkflowsClient
from .types.workflows_execute_workflow_response import WorkflowsExecuteWorkflowResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class WorkflowsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawWorkflowsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawWorkflowsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawWorkflowsClient
        """
        return self._raw_client

    def execute_workflow(
        self,
        application_id: str,
        *,
        inputs: typing.Optional[typing.Dict[str, typing.Any]] = OMIT,
        inputs_literal_map: typing.Optional[typing.Dict[str, typing.Any]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> WorkflowsExecuteWorkflowResponse:
        """
        Execute a workflow for the specified application

        Parameters
        ----------
        application_id : str
            Id of the application

        inputs : typing.Optional[typing.Dict[str, typing.Any]]
            Workflow inputs

        inputs_literal_map : typing.Optional[typing.Dict[str, typing.Any]]
            Workflow inputs literal map

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        WorkflowsExecuteWorkflowResponse
            Returns execution name of the workflow

        Examples
        --------
        from truefoundry_sdk import TrueFoundry

        client = TrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        client.internal.workflows.execute_workflow(
            application_id="applicationId",
        )
        """
        _response = self._raw_client.execute_workflow(
            application_id, inputs=inputs, inputs_literal_map=inputs_literal_map, request_options=request_options
        )
        return _response.data


class AsyncWorkflowsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawWorkflowsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawWorkflowsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawWorkflowsClient
        """
        return self._raw_client

    async def execute_workflow(
        self,
        application_id: str,
        *,
        inputs: typing.Optional[typing.Dict[str, typing.Any]] = OMIT,
        inputs_literal_map: typing.Optional[typing.Dict[str, typing.Any]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> WorkflowsExecuteWorkflowResponse:
        """
        Execute a workflow for the specified application

        Parameters
        ----------
        application_id : str
            Id of the application

        inputs : typing.Optional[typing.Dict[str, typing.Any]]
            Workflow inputs

        inputs_literal_map : typing.Optional[typing.Dict[str, typing.Any]]
            Workflow inputs literal map

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        WorkflowsExecuteWorkflowResponse
            Returns execution name of the workflow

        Examples
        --------
        import asyncio

        from truefoundry_sdk import AsyncTrueFoundry

        client = AsyncTrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.internal.workflows.execute_workflow(
                application_id="applicationId",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.execute_workflow(
            application_id, inputs=inputs, inputs_literal_map=inputs_literal_map, request_options=request_options
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core import enum

T_Result = typing.TypeVar("T_Result")


class JobRunsSortBy(enum.StrEnum):
    START_TIME = "startTime"
    DURATION = "duration"
    DEPLOYMENT_VERSION = "deploymentVersion"
    TRIGGERED_BY = "triggeredBy"
    STATUS = "status"

    def visit(
        self,
        start_time: typing.Callable[[], T_Result],
        duration: typing.Callable[[], T_Result],
        deployment_version: typing.Callable[[], T_Result],
        triggered_by: typing.Callable[[], T_Result],
        status: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is JobRunsSortBy.START_TIME:
            return start_time()
        if self is JobRunsSortBy.DURATION:
            return duration()
        if self is JobRunsSortBy.DEPLOYMENT_VERSION:
            return deployment_version()
        if self is JobRunsSortBy.TRIGGERED_BY:
            return triggered_by()
        if self is JobRunsSortBy.STATUS:
            return status()



# This file was auto-generated by Fern from our API Definition.

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.request_options import RequestOptions
from ...types.gateway_configuration import GatewayConfiguration
from .raw_client import AsyncRawAiGatewayClient, RawAiGatewayClient
from .types.ai_gateway_get_gateway_config_request_type import AiGatewayGetGatewayConfigRequestType


class AiGatewayClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawAiGatewayClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawAiGatewayClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawAiGatewayClient
        """
        return self._raw_client

    def get_gateway_config(
        self, type: AiGatewayGetGatewayConfigRequestType, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GatewayConfiguration:
        """
        Get Gateway configuration based on type for the tenant.

        Parameters
        ----------
        type : AiGatewayGetGatewayConfigRequestType
            Type of Config

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GatewayConfiguration
            Gateway configuration retrieved successfully

        Examples
        --------
        from truefoundry_sdk import TrueFoundry
        from truefoundry_sdk.internal.ai_gateway import (
            AiGatewayGetGatewayConfigRequestType,
        )

        client = TrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        client.internal.ai_gateway.get_gateway_config(
            type=AiGatewayGetGatewayConfigRequestType.GATEWAY_RATE_LIMITING_CONFIG,
        )
        """
        _response = self._raw_client.get_gateway_config(type, request_options=request_options)
        return _response.data


class AsyncAiGatewayClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawAiGatewayClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawAiGatewayClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawAiGatewayClient
        """
        return self._raw_client

    async def get_gateway_config(
        self, type: AiGatewayGetGatewayConfigRequestType, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GatewayConfiguration:
        """
        Get Gateway configuration based on type for the tenant.

        Parameters
        ----------
        type : AiGatewayGetGatewayConfigRequestType
            Type of Config

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GatewayConfiguration
            Gateway configuration retrieved successfully

        Examples
        --------
        import asyncio

        from truefoundry_sdk import AsyncTrueFoundry
        from truefoundry_sdk.internal.ai_gateway import (
            AiGatewayGetGatewayConfigRequestType,
        )

        client = AsyncTrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.internal.ai_gateway.get_gateway_config(
                type=AiGatewayGetGatewayConfigRequestType.GATEWAY_RATE_LIMITING_CONFIG,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_gateway_config(type, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .mcp_server_header_override_auth import McpServerHeaderOverrideAuth


class VirtualMcpServerSource(UniversalBaseModel):
    """
    Virtual MCP Server Source
    """

    name: str = pydantic.Field()
    """
    Name of the source MCP server within this group (use {provider_account_name}/{integration_name} format).
    """

    enabled_tools: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    List of tools to enable from this source server.
    """

    auth_data: typing.Optional[McpServerHeaderOverrideAuth] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .sklearn_model_schema import SklearnModelSchema
from .sklearn_serialization_format import SklearnSerializationFormat


class SklearnFramework(UniversalBaseModel):
    type: typing.Literal["sklearn"] = pydantic.Field(default="sklearn")
    """
    Type of the framework
    """

    model_filepath: typing.Optional[str] = pydantic.Field(default=None)
    """
    Relative path to the model file in the model version contents
    """

    serialization_format: typing.Optional[SklearnSerializationFormat] = None
    model_schema: typing.Optional[SklearnModelSchema] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .gateway_otel_config_otel_traces_exporter_config import GatewayOtelConfigOtelTracesExporterConfig


class GatewayOtelConfig(UniversalBaseModel):
    type: typing.Literal["gateway-otel-config"] = pydantic.Field(default="gateway-otel-config")
    """
    +value=gateway-otel-config
    +sort=1
    """

    otel_traces_exporter_config: typing.Optional[GatewayOtelConfigOtelTracesExporterConfig] = pydantic.Field(
        default=None
    )
    """
    Set this configuration to export LLM gateway OTEL traces to an external platform. Note that we only export OTEL traces via this configuration, this does not include OTEL metrics.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class PolicyMutationOperation(UniversalBaseModel):
    """
    Mutate
    """

    type: typing.Literal["mutate"] = pydantic.Field(default="mutate")
    """
    Mutates or Validates Resources
    """

    order: int = pydantic.Field()
    """
    Determines the sequence in which mutation policies are executed. Policies with lower order value run first, followed by higher value. For example, a policy with order 10 runs before one with order 20. This is crucial when multiple mutation policies need to run in a specific sequence. The order must be between 1 and 100.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core import enum

T_Result = typing.TypeVar("T_Result")


class GcpRegion(enum.StrEnum):
    GLOBAL = "global"
    NORTHAMERICA_NORTHEAST1 = "northamerica-northeast1"
    NORTHAMERICA_NORTHEAST2 = "northamerica-northeast2"
    SOUTHAMERICA_EAST1 = "southamerica-east1"
    US_CENTRAL1 = "us-central1"
    US_EAST1 = "us-east1"
    US_EAST4 = "us-east4"
    US_EAST5 = "us-east5"
    US_SOUTH1 = "us-south1"
    US_WEST1 = "us-west1"
    US_WEST2 = "us-west2"
    US_WEST3 = "us-west3"
    US_WEST4 = "us-west4"
    ASIA_EAST1 = "asia-east1"
    ASIA_EAST2 = "asia-east2"
    ASIA_NORTHEAST1 = "asia-northeast1"
    ASIA_NORTHEAST2 = "asia-northeast2"
    ASIA_NORTHEAST3 = "asia-northeast3"
    ASIA_SOUTH1 = "asia-south1"
    ASIA_SOUTH2 = "asia-south2"
    ASIA_SOUTHEAST1 = "asia-southeast1"
    ASIA_SOUTHEAST2 = "asia-southeast2"
    AUSTRALIA_SOUTHEAST1 = "australia-southeast1"
    AUSTRALIA_SOUTHEAST2 = "australia-southeast2"
    EUROPE_CENTRAL2 = "europe-central2"
    EUROPE_NORTH1 = "europe-north1"
    EUROPE_SOUTHWEST1 = "europe-southwest1"
    EUROPE_WEST1 = "europe-west1"
    EUROPE_WEST2 = "europe-west2"
    EUROPE_WEST3 = "europe-west3"
    EUROPE_WEST4 = "europe-west4"
    EUROPE_WEST6 = "europe-west6"
    EUROPE_WEST8 = "europe-west8"
    EUROPE_WEST9 = "europe-west9"
    ME_CENTRAL1 = "me-central1"
    ME_CENTRAL2 = "me-central2"
    ME_WEST1 = "me-west1"

    def visit(
        self,
        global_: typing.Callable[[], T_Result],
        northamerica_northeast1: typing.Callable[[], T_Result],
        northamerica_northeast2: typing.Callable[[], T_Result],
        southamerica_east1: typing.Callable[[], T_Result],
        us_central1: typing.Callable[[], T_Result],
        us_east1: typing.Callable[[], T_Result],
        us_east4: typing.Callable[[], T_Result],
        us_east5: typing.Callable[[], T_Result],
        us_south1: typing.Callable[[], T_Result],
        us_west1: typing.Callable[[], T_Result],
        us_west2: typing.Callable[[], T_Result],
        us_west3: typing.Callable[[], T_Result],
        us_west4: typing.Callable[[], T_Result],
        asia_east1: typing.Callable[[], T_Result],
        asia_east2: typing.Callable[[], T_Result],
        asia_northeast1: typing.Callable[[], T_Result],
        asia_northeast2: typing.Callable[[], T_Result],
        asia_northeast3: typing.Callable[[], T_Result],
        asia_south1: typing.Callable[[], T_Result],
        asia_south2: typing.Callable[[], T_Result],
        asia_southeast1: typing.Callable[[], T_Result],
        asia_southeast2: typing.Callable[[], T_Result],
        australia_southeast1: typing.Callable[[], T_Result],
        australia_southeast2: typing.Callable[[], T_Result],
        europe_central2: typing.Callable[[], T_Result],
        europe_north1: typing.Callable[[], T_Result],
        europe_southwest1: typing.Callable[[], T_Result],
        europe_west1: typing.Callable[[], T_Result],
        europe_west2: typing.Callable[[], T_Result],
        europe_west3: typing.Callable[[], T_Result],
        europe_west4: typing.Callable[[], T_Result],
        europe_west6: typing.Callable[[], T_Result],
        europe_west8: typing.Callable[[], T_Result],
        europe_west9: typing.Callable[[], T_Result],
        me_central1: typing.Callable[[], T_Result],
        me_central2: typing.Callable[[], T_Result],
        me_west1: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is GcpRegion.GLOBAL:
            return global_()
        if self is GcpRegion.NORTHAMERICA_NORTHEAST1:
            return northamerica_northeast1()
        if self is GcpRegion.NORTHAMERICA_NORTHEAST2:
            return northamerica_northeast2()
        if self is GcpRegion.SOUTHAMERICA_EAST1:
            return southamerica_east1()
        if self is GcpRegion.US_CENTRAL1:
            return us_central1()
        if self is GcpRegion.US_EAST1:
            return us_east1()
        if self is GcpRegion.US_EAST4:
            return us_east4()
        if self is GcpRegion.US_EAST5:
            return us_east5()
        if self is GcpRegion.US_SOUTH1:
            return us_south1()
        if self is GcpRegion.US_WEST1:
            return us_west1()
        if self is GcpRegion.US_WEST2:
            return us_west2()
        if self is GcpRegion.US_WEST3:
            return us_west3()
        if self is GcpRegion.US_WEST4:
            return us_west4()
        if self is GcpRegion.ASIA_EAST1:
            return asia_east1()
        if self is GcpRegion.ASIA_EAST2:
            return asia_east2()
        if self is GcpRegion.ASIA_NORTHEAST1:
            return asia_northeast1()
        if self is GcpRegion.ASIA_NORTHEAST2:
            return asia_northeast2()
        if self is GcpRegion.ASIA_NORTHEAST3:
            return asia_northeast3()
        if self is GcpRegion.ASIA_SOUTH1:
            return asia_south1()
        if self is GcpRegion.ASIA_SOUTH2:
            return asia_south2()
        if self is GcpRegion.ASIA_SOUTHEAST1:
            return asia_southeast1()
        if self is GcpRegion.ASIA_SOUTHEAST2:
            return asia_southeast2()
        if self is GcpRegion.AUSTRALIA_SOUTHEAST1:
            return australia_southeast1()
        if self is GcpRegion.AUSTRALIA_SOUTHEAST2:
            return australia_southeast2()
        if self is GcpRegion.EUROPE_CENTRAL2:
            return europe_central2()
        if self is GcpRegion.EUROPE_NORTH1:
            return europe_north1()
        if self is GcpRegion.EUROPE_SOUTHWEST1:
            return europe_southwest1()
        if self is GcpRegion.EUROPE_WEST1:
            return europe_west1()
        if self is GcpRegion.EUROPE_WEST2:
            return europe_west2()
        if self is GcpRegion.EUROPE_WEST3:
            return europe_west3()
        if self is GcpRegion.EUROPE_WEST4:
            return europe_west4()
        if self is GcpRegion.EUROPE_WEST6:
            return europe_west6()
        if self is GcpRegion.EUROPE_WEST8:
            return europe_west8()
        if self is GcpRegion.EUROPE_WEST9:
            return europe_west9()
        if self is GcpRegion.ME_CENTRAL1:
            return me_central1()
        if self is GcpRegion.ME_CENTRAL2:
            return me_central2()
        if self is GcpRegion.ME_WEST1:
            return me_west1()



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core import enum

T_Result = typing.TypeVar("T_Result")


class AlertSeverity(enum.StrEnum):
    WARNING = "warning"
    CRITICAL = "critical"

    def visit(self, warning: typing.Callable[[], T_Result], critical: typing.Callable[[], T_Result]) -> T_Result:
        if self is AlertSeverity.WARNING:
            return warning()
        if self is AlertSeverity.CRITICAL:
            return critical()



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class GitSource(UniversalBaseModel):
    """
    Describes that we are using code stored in a git repository to build our image
    """

    type: typing.Literal["git"] = pydantic.Field(default="git")
    """
    +value=git
    """

    repo_url: str = pydantic.Field()
    """
    The repository URL.
    """

    ref: str = pydantic.Field()
    """
    The commit SHA.
    """

    branch_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Selecting branch will select latest commit SHA of the branch.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .spark_build import SparkBuild
from .spark_image_build_build_source import SparkImageBuildBuildSource


class SparkImageBuild(UniversalBaseModel):
    """
    Describes that we are building a new image based on the spec
    """

    type: typing.Literal["spark-image-build"] = pydantic.Field(default="spark-image-build")
    """
    +value=spark-image-build
    """

    docker_registry: typing.Optional[str] = pydantic.Field(default=None)
    """
    FQN of the container registry. If you can't find your registry here,
    add it through the [Integrations](/integrations?tab=docker-registry) page
    """

    build_source: SparkImageBuildBuildSource = pydantic.Field()
    """
    Fetch source code
    """

    build_spec: SparkBuild

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.request_options import RequestOptions
from ...types.get_authenticated_vcsurl_response import GetAuthenticatedVcsurlResponse
from ...types.git_repository_exists_response import GitRepositoryExistsResponse
from .raw_client import AsyncRawVcsClient, RawVcsClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class VcsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawVcsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawVcsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawVcsClient
        """
        return self._raw_client

    def get_repository_details(
        self, *, repo_url: str, id: typing.Optional[str] = OMIT, request_options: typing.Optional[RequestOptions] = None
    ) -> GitRepositoryExistsResponse:
        """
        Parameters
        ----------
        repo_url : str
            The URL of the repository

        id : typing.Optional[str]
            The integration id of the repository

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GitRepositoryExistsResponse
            Returns git repository details if git repository exists and is accessible

        Examples
        --------
        from truefoundry_sdk import TrueFoundry

        client = TrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        client.internal.vcs.get_repository_details(
            repo_url="repoURL",
        )
        """
        _response = self._raw_client.get_repository_details(repo_url=repo_url, id=id, request_options=request_options)
        return _response.data

    def get_authenticated_url(
        self, *, repo_url: str, request_options: typing.Optional[RequestOptions] = None
    ) -> GetAuthenticatedVcsurlResponse:
        """
        Parameters
        ----------
        repo_url : str
            Repository URL (e.g., https://github.com/user/repo, https://bitbucket.org/user/repo)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetAuthenticatedVcsurlResponse
            Returns an authenticated VCS URL for the repository

        Examples
        --------
        from truefoundry_sdk import TrueFoundry

        client = TrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )
        client.internal.vcs.get_authenticated_url(
            repo_url="repoURL",
        )
        """
        _response = self._raw_client.get_authenticated_url(repo_url=repo_url, request_options=request_options)
        return _response.data


class AsyncVcsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawVcsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawVcsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawVcsClient
        """
        return self._raw_client

    async def get_repository_details(
        self, *, repo_url: str, id: typing.Optional[str] = OMIT, request_options: typing.Optional[RequestOptions] = None
    ) -> GitRepositoryExistsResponse:
        """
        Parameters
        ----------
        repo_url : str
            The URL of the repository

        id : typing.Optional[str]
            The integration id of the repository

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GitRepositoryExistsResponse
            Returns git repository details if git repository exists and is accessible

        Examples
        --------
        import asyncio

        from truefoundry_sdk import AsyncTrueFoundry

        client = AsyncTrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.internal.vcs.get_repository_details(
                repo_url="repoURL",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_repository_details(
            repo_url=repo_url, id=id, request_options=request_options
        )
        return _response.data

    async def get_authenticated_url(
        self, *, repo_url: str, request_options: typing.Optional[RequestOptions] = None
    ) -> GetAuthenticatedVcsurlResponse:
        """
        Parameters
        ----------
        repo_url : str
            Repository URL (e.g., https://github.com/user/repo, https://bitbucket.org/user/repo)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetAuthenticatedVcsurlResponse
            Returns an authenticated VCS URL for the repository

        Examples
        --------
        import asyncio

        from truefoundry_sdk import AsyncTrueFoundry

        client = AsyncTrueFoundry(
            api_key="YOUR_API_KEY",
            base_url="https://yourhost.com/path/to/api",
        )


        async def main() -> None:
            await client.internal.vcs.get_authenticated_url(
                repo_url="repoURL",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_authenticated_url(repo_url=repo_url, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class TrueFoundryArtifactSource(UniversalBaseModel):
    """
    Input for Artifact from TrueFoundry Artifact Registry
    """

    type: typing.Literal["truefoundry-artifact"] = pydantic.Field(default="truefoundry-artifact")
    """
    +value=truefoundry-artifact
    """

    artifact_version_fqn: str = pydantic.Field()
    """
    Artifact or Model Version FQN of the artifact to be downloaded
    """

    download_path_env_variable: str = pydantic.Field()
    """
    Environment variable which will contain the download path of the artifact
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from .mcp_server_with_fqn import McpServerWithFqn
from .mcp_server_with_url import McpServerWithUrl

ChatPromptManifestMcpServersItem = typing.Union[McpServerWithFqn, McpServerWithUrl]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class DynamicVolumeConfig(UniversalBaseModel):
    """
    Create new volume
    """

    type: typing.Literal["dynamic"] = pydantic.Field(default="dynamic")
    """
    Volume Type for the volume.
    """

    storage_class: str = pydantic.Field()
    """
    Name of the storage class to be used for the volume.
    """

    size: int = pydantic.Field()
    """
    Size of volume in Gi
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core import enum

T_Result = typing.TypeVar("T_Result")


class PatronusGliderCriteria(enum.StrEnum):
    PATRONUS_IS_COMPLIANT = "patronus:is-compliant"
    PATRONUS_IS_FACTUALLY_CONSISTENT = "patronus:is-factually-consistent"
    PATRONUS_IS_GOOD_SUMMARY = "patronus:is-good-summary"
    PATRONUS_IS_HARMFUL_ADVICE = "patronus:is-harmful-advice"
    PATRONUS_IS_INFORMAL_TONE = "patronus:is-informal-tone"

    def visit(
        self,
        patronus_is_compliant: typing.Callable[[], T_Result],
        patronus_is_factually_consistent: typing.Callable[[], T_Result],
        patronus_is_good_summary: typing.Callable[[], T_Result],
        patronus_is_harmful_advice: typing.Callable[[], T_Result],
        patronus_is_informal_tone: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is PatronusGliderCriteria.PATRONUS_IS_COMPLIANT:
            return patronus_is_compliant()
        if self is PatronusGliderCriteria.PATRONUS_IS_FACTUALLY_CONSISTENT:
            return patronus_is_factually_consistent()
        if self is PatronusGliderCriteria.PATRONUS_IS_GOOD_SUMMARY:
            return patronus_is_good_summary()
        if self is PatronusGliderCriteria.PATRONUS_IS_HARMFUL_ADVICE:
            return patronus_is_harmful_advice()
        if self is PatronusGliderCriteria.PATRONUS_IS_INFORMAL_TONE:
            return patronus_is_informal_tone()



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pagination import AsyncPager, SyncPager
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..core.serialization import convert_and_respect_annotation_metadata
from ..errors.bad_request_error import BadRequestError
from ..errors.conflict_error import ConflictError
from ..errors.failed_dependency_error import FailedDependencyError
from ..errors.forbidden_error import ForbiddenError
from ..errors.not_found_error import NotFoundError
from ..errors.unprocessable_entity_error import UnprocessableEntityError
from ..types.delete_secret_group_response import DeleteSecretGroupResponse
from ..types.get_secret_group_response import GetSecretGroupResponse
from ..types.http_error import HttpError
from ..types.list_secret_group_response import ListSecretGroupResponse
from ..types.secret_group import SecretGroup
from ..types.secret_group_manifest import SecretGroupManifest
from ..types.secret_input import SecretInput
from ..types.update_secret_input import UpdateSecretInput

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawSecretGroupsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list(
        self,
        *,
        limit: typing.Optional[int] = 100,
        offset: typing.Optional[int] = 0,
        fqn: typing.Optional[str] = None,
        search: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[SecretGroup, ListSecretGroupResponse]:
        """
        List the secret groups associated with a user along with the associated secrets for each group. Filtered with the options passed in the query fields. Note: This method does not return the secret values of the <em>associatedSecrets</em> in the response. A separate API call to `/v1/secrets/{id}` should be made to fetch the associated secret value.

        Parameters
        ----------
        limit : typing.Optional[int]
            Number of items per page

        offset : typing.Optional[int]
            Number of items to skip

        fqn : typing.Optional[str]
            Fqn of secret group.

        search : typing.Optional[str]
            Search query - filters by secret group names that contain the search string

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[SecretGroup, ListSecretGroupResponse]
            Returns all the secret groups associated with a user along with the associated secrets for each group.
        """
        offset = offset if offset is not None else 0

        _response = self._client_wrapper.httpx_client.request(
            "api/svc/v1/secret-groups",
            method="GET",
            params={
                "limit": limit,
                "offset": offset,
                "fqn": fqn,
                "search": search,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListSecretGroupResponse,
                    parse_obj_as(
                        type_=ListSecretGroupResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _items = _parsed_response.data
                _has_next = True
                _get_next = lambda: self.list(
                    limit=limit,
                    offset=offset + len(_items),
                    fqn=fqn,
                    search=search,
                    request_options=request_options,
                )
                return SyncPager(has_next=_has_next, items=_items, get_next=_get_next, response=_parsed_response)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def create(
        self,
        *,
        name: str,
        integration_id: str,
        secrets: typing.Sequence[SecretInput],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[GetSecretGroupResponse]:
        """
        Creates a secret group with secrets in it. A secret version for each of the created secret is created with version number as 1. The returned secret group does not have any secret values in the <em>associatedSecrets</em> field. A separate API call to `/v1/secrets/{id}` should be made to fetch the associated secret value.

        Parameters
        ----------
        name : str
            Name of the secret group.

        integration_id : str
            Id of the provider integration.

        secrets : typing.Sequence[SecretInput]
            The secrets to be associated with the secret group

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetSecretGroupResponse]
            Returns the created secret group without the associated secrets.
        """
        _response = self._client_wrapper.httpx_client.request(
            "api/svc/v1/secret-groups",
            method="POST",
            json={
                "name": name,
                "integrationId": integration_id,
                "secrets": convert_and_respect_annotation_metadata(
                    object_=secrets, annotation=typing.Sequence[SecretInput], direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetSecretGroupResponse,
                    parse_obj_as(
                        type_=GetSecretGroupResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 424:
                raise FailedDependencyError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpError,
                        parse_obj_as(
                            type_=HttpError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def create_or_update(
        self, *, manifest: SecretGroupManifest, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[GetSecretGroupResponse]:
        """
        Creates a new secret group or updates an existing one based on the provided manifest.

        Parameters
        ----------
        manifest : SecretGroupManifest
            Secret Group Manifest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetSecretGroupResponse]
            Secret group created or updated successfully.
        """
        _response = self._client_wrapper.httpx_client.request(
            "api/svc/v1/secret-groups",
            method="PUT",
            json={
                "manifest": convert_and_respect_annotation_metadata(
                    object_=manifest, annotation=SecretGroupManifest, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetSecretGroupResponse,
                    parse_obj_as(
                        type_=GetSecretGroupResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpError,
                        parse_obj_as(
                            type_=HttpError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpError,
                        parse_obj_as(
                            type_=HttpError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[GetSecretGroupResponse]:
        """
        Get Secret Group by id. This method does not return the secret values of the <em>associatedSecrets</em> in the response. A separate API call to `/v1/secrets/{id}` should be made to fetch the associated secret value.

        Parameters
        ----------
        id : str
            Secret Id of the secret group.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetSecretGroupResponse]
            Returns the Secret Group associated with provided id
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/svc/v1/secret-groups/{jsonable_encoder(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetSecretGroupResponse,
                    parse_obj_as(
                        type_=GetSecretGroupResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpError,
                        parse_obj_as(
                            type_=HttpError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update(
        self,
        id: str,
        *,
        secrets: typing.Sequence[UpdateSecretInput],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[GetSecretGroupResponse]:
        """
        Updates the secrets in a secret group with new values. A new secret version is created for every secret that has a modified value and any omitted secrets are deleted. The returned updated secret group does not have any secret values in the <em>associatedSecrets</em> field. A separate API call to `/v1/secrets/{id}` should be made to fetch the associated secret value.

        Parameters
        ----------
        id : str
            Secret Id of the secret group.

        secrets : typing.Sequence[UpdateSecretInput]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetSecretGroupResponse]
            Returns the updated secret group without associated secrets.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/svc/v1/secret-groups/{jsonable_encoder(id)}",
            method="PUT",
            json={
                "secrets": convert_and_respect_annotation_metadata(
                    object_=secrets, annotation=typing.Sequence[UpdateSecretInput], direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetSecretGroupResponse,
                    parse_obj_as(
                        type_=GetSecretGroupResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpError,
                        parse_obj_as(
                            type_=HttpError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def delete(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[DeleteSecretGroupResponse]:
        """
        Deletes the secret group, its associated secrets and secret versions of those secrets.

        Parameters
        ----------
        id : str
            Secret Id of the secret group.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[DeleteSecretGroupResponse]
            Deletes Secret Group.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/svc/v1/secret-groups/{jsonable_encoder(id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DeleteSecretGroupResponse,
                    parse_obj_as(
                        type_=DeleteSecretGroupResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpError,
                        parse_obj_as(
                            type_=HttpError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawSecretGroupsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list(
        self,
        *,
        limit: typing.Optional[int] = 100,
        offset: typing.Optional[int] = 0,
        fqn: typing.Optional[str] = None,
        search: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[SecretGroup, ListSecretGroupResponse]:
        """
        List the secret groups associated with a user along with the associated secrets for each group. Filtered with the options passed in the query fields. Note: This method does not return the secret values of the <em>associatedSecrets</em> in the response. A separate API call to `/v1/secrets/{id}` should be made to fetch the associated secret value.

        Parameters
        ----------
        limit : typing.Optional[int]
            Number of items per page

        offset : typing.Optional[int]
            Number of items to skip

        fqn : typing.Optional[str]
            Fqn of secret group.

        search : typing.Optional[str]
            Search query - filters by secret group names that contain the search string

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[SecretGroup, ListSecretGroupResponse]
            Returns all the secret groups associated with a user along with the associated secrets for each group.
        """
        offset = offset if offset is not None else 0

        _response = await self._client_wrapper.httpx_client.request(
            "api/svc/v1/secret-groups",
            method="GET",
            params={
                "limit": limit,
                "offset": offset,
                "fqn": fqn,
                "search": search,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListSecretGroupResponse,
                    parse_obj_as(
                        type_=ListSecretGroupResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _items = _parsed_response.data
                _has_next = True

                async def _get_next():
                    return await self.list(
                        limit=limit,
                        offset=offset + len(_items),
                        fqn=fqn,
                        search=search,
                        request_options=request_options,
                    )

                return AsyncPager(has_next=_has_next, items=_items, get_next=_get_next, response=_parsed_response)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def create(
        self,
        *,
        name: str,
        integration_id: str,
        secrets: typing.Sequence[SecretInput],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[GetSecretGroupResponse]:
        """
        Creates a secret group with secrets in it. A secret version for each of the created secret is created with version number as 1. The returned secret group does not have any secret values in the <em>associatedSecrets</em> field. A separate API call to `/v1/secrets/{id}` should be made to fetch the associated secret value.

        Parameters
        ----------
        name : str
            Name of the secret group.

        integration_id : str
            Id of the provider integration.

        secrets : typing.Sequence[SecretInput]
            The secrets to be associated with the secret group

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetSecretGroupResponse]
            Returns the created secret group without the associated secrets.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "api/svc/v1/secret-groups",
            method="POST",
            json={
                "name": name,
                "integrationId": integration_id,
                "secrets": convert_and_respect_annotation_metadata(
                    object_=secrets, annotation=typing.Sequence[SecretInput], direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetSecretGroupResponse,
                    parse_obj_as(
                        type_=GetSecretGroupResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 424:
                raise FailedDependencyError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpError,
                        parse_obj_as(
                            type_=HttpError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def create_or_update(
        self, *, manifest: SecretGroupManifest, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[GetSecretGroupResponse]:
        """
        Creates a new secret group or updates an existing one based on the provided manifest.

        Parameters
        ----------
        manifest : SecretGroupManifest
            Secret Group Manifest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetSecretGroupResponse]
            Secret group created or updated successfully.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "api/svc/v1/secret-groups",
            method="PUT",
            json={
                "manifest": convert_and_respect_annotation_metadata(
                    object_=manifest, annotation=SecretGroupManifest, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetSecretGroupResponse,
                    parse_obj_as(
                        type_=GetSecretGroupResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpError,
                        parse_obj_as(
                            type_=HttpError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpError,
                        parse_obj_as(
                            type_=HttpError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[GetSecretGroupResponse]:
        """
        Get Secret Group by id. This method does not return the secret values of the <em>associatedSecrets</em> in the response. A separate API call to `/v1/secrets/{id}` should be made to fetch the associated secret value.

        Parameters
        ----------
        id : str
            Secret Id of the secret group.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetSecretGroupResponse]
            Returns the Secret Group associated with provided id
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/svc/v1/secret-groups/{jsonable_encoder(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetSecretGroupResponse,
                    parse_obj_as(
                        type_=GetSecretGroupResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpError,
                        parse_obj_as(
                            type_=HttpError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update(
        self,
        id: str,
        *,
        secrets: typing.Sequence[UpdateSecretInput],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[GetSecretGroupResponse]:
        """
        Updates the secrets in a secret group with new values. A new secret version is created for every secret that has a modified value and any omitted secrets are deleted. The returned updated secret group does not have any secret values in the <em>associatedSecrets</em> field. A separate API call to `/v1/secrets/{id}` should be made to fetch the associated secret value.

        Parameters
        ----------
        id : str
            Secret Id of the secret group.

        secrets : typing.Sequence[UpdateSecretInput]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetSecretGroupResponse]
            Returns the updated secret group without associated secrets.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/svc/v1/secret-groups/{jsonable_encoder(id)}",
            method="PUT",
            json={
                "secrets": convert_and_respect_annotation_metadata(
                    object_=secrets, annotation=typing.Sequence[UpdateSecretInput], direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetSecretGroupResponse,
                    parse_obj_as(
                        type_=GetSecretGroupResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpError,
                        parse_obj_as(
                            type_=HttpError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def delete(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[DeleteSecretGroupResponse]:
        """
        Deletes the secret group, its associated secrets and secret versions of those secrets.

        Parameters
        ----------
        id : str
            Secret Id of the secret group.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[DeleteSecretGroupResponse]
            Deletes Secret Group.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/svc/v1/secret-groups/{jsonable_encoder(id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DeleteSecretGroupResponse,
                    parse_obj_as(
                        type_=DeleteSecretGroupResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpError,
                        parse_obj_as(
                            type_=HttpError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .load_balance_target import LoadBalanceTarget


class WeightBasedLoadBalancing(UniversalBaseModel):
    """
    Weight-based Load Balancing
    """

    type: typing.Literal["weight-based-routing"] = pydantic.Field(default="weight-based-routing")
    """
    Routing Type
    """

    load_balance_targets: typing.List[LoadBalanceTarget] = pydantic.Field()
    """
    List of targets for load balancing with weights
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .pagination import Pagination
from .prompt import Prompt


class ListPromptsResponse(UniversalBaseModel):
    data: typing.List[Prompt]
    pagination: Pagination

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class SparkJobJavaEntrypoint(UniversalBaseModel):
    """
    java
    """

    type: typing.Literal["java"] = pydantic.Field(default="java")
    """
    +value=java
    """

    main_application_file: str = pydantic.Field()
    """
    The main application file to be executed by the spark job.
    """

    main_class: str = pydantic.Field()
    """
    The main class to be executed by the spark job.
    """

    arguments: typing.Optional[str] = pydantic.Field(default=None)
    """
    Arguments to be passed to the main application file.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class AsyncProcessorSidecar(UniversalBaseModel):
    destination_url: str = pydantic.Field()
    """
    URL for the processor to invoke
    """

    request_timeout: typing.Optional[int] = pydantic.Field(default=10)
    """
    Timeout for the invoke request in seconds
    """

    sidecar_image: typing.Optional[str] = pydantic.Field(default=None)
    """
    Image for the processor sidecar (This field will be deprecated in the future)
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow")  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ....types.artifact_manifest import ArtifactManifest
from ....types.chat_prompt_manifest import ChatPromptManifest
from ....types.data_directory_manifest import DataDirectoryManifest
from ....types.model_manifest import ModelManifest
from ....types.tracing_project_manifest import TracingProjectManifest

ApplyMlEntityRequestManifest = typing.Union[
    ModelManifest, ChatPromptManifest, ArtifactManifest, DataDirectoryManifest, TracingProjectManifest
]
